clear all
clc;

%% Import Data from Notepad

% Read result from decay test
foldername = 'F:\MSc Marine Technology NTNU\3rd Semester\TMR03 - Integrated Dynamic Analysis of Floating Wind Turbines\SIMA_Matlab Codes\Matlab Routines\Free Decay';
surgename = [foldername '\Surge.txt'];
heavename = [foldername '\Heave.txt'];
swayname = [foldername '\Sway.txt'];
pitchname = [foldername '\Pitch.txt'];
rollname = [foldername '\Roll.txt'];
yawname = [foldername '\Yaw.txt'];

% Motion Time Histories
n1 = importdata(surgename).data.';     %Surge Motion History
n2 = importdata(swayname).data.';      %Sway Motion History
n3 = importdata(heavename).data.';     %Heave Motion History
n4 = importdata(rollname).data.';      %Roll Motion History
n5 = importdata(pitchname).data.';     %Pitch Motion History
n6 = importdata(yawname).data.';       %Yaw Motion History

% Time Range
t1 = (size(n1,2)/10:0.1:size(n1,1)/10).';   %Surge Time series
t2 = (size(n2,2)/10:0.1:size(n2,1)/10).';   %Sway Time series
t3 = (size(n3,2)/10:0.1:size(n3,1)/10).';   %Heave Time series
t4 = (size(n4,2)/10:0.1:size(n4,1)/10).';   %Roll Time series
t5 = (size(n5,2)/10:0.1:size(n5,1)/10).';   %Pitch Time series
t6 = (size(n6,2)/10:0.1:size(n6,1)/10).';   %Yaw Time series

plot(t1,n1)

 %% PQ Analysis

% Statistical anlysis carried out after t = 250s
t_start = 250;
ind_surge = sum(t1<=t_start);
ind_sway = sum(t2<=t_start);
ind_heave = sum(t3<=t_start);
ind_roll = sum(t4<=t_start);
ind_pitch = sum(t5<=t_start);
ind_yaw = sum(t6<=t_start);

% Time Histories with modified range
t1_new = t1(ind_surge+1:end);
t2_new = t2(ind_sway+1:end);
t3_new = t3(ind_heave+50:end);
t4_new = t4(ind_roll+100:end-50);
t5_new = t5(ind_pitch+1:end-50);
t6_new = t6(ind_yaw+1:end);

% Motion Histories with modified range
n1_new = n1(ind_surge+1:end);
n2_new = n2(ind_sway+1:end);
n3_new = n3(ind_heave+50:end);
n4_new = n4(ind_roll+100:end-50);
n5_new = n5(ind_pitch+1:end-50);
n6_new = n6(ind_yaw+1:end);

%% As per Vishnu's Range

% % t_start = 250;
% % ind_surge = sum(t1<=t_start);
% % ind_sway = sum(t2<=t_start);
% % ind_heave = sum(t3<=t_start);
% % ind_roll = sum(t4<=t_start);
% % ind_pitch = sum(t5<=t_start);
% % ind_yaw = sum(t6<=t_start);
% 
% 
% ind_surge = sum(t1<=250);
% ind_sway = sum(t2<=250);
% ind_heave = sum(t3<=210);
% ind_roll = sum(t4<=250);
% ind_pitch = sum(t5<=250);
% ind_yaw = sum(t6<=220);
% 
% % Time Histories with modified range
% t1_new = t1(ind_surge:end);
% t2_new = t2(ind_sway:end);
% t3_new = t3(ind_heave:end-50);
% t4_new = t4(ind_roll:end-2500);
% t5_new = t5(ind_pitch:end-2500);
% t6_new = t6(ind_yaw:end-10);
% 
% % Motion Histories with modified range
% n1_new = n1(ind_surge:end);
% n2_new = n2(ind_sway:end);
% n3_new = n3(ind_heave:end-50);
% n4_new = n4(ind_roll:end-2500);
% n5_new = n5(ind_pitch:end-2500);
% n6_new = n6(ind_yaw:end-10);
%%

% Surge
[P1,Q1,fighandle1,xbar1,dx1,tp1,tpp1] = PQanalysis(t1_new,n1_new,1);

% Sway
[P2,Q2,fighandle2,xbar2,dx2,tp2,tpp2] = PQanalysis(t2_new,n2_new,1);

% Heave
[P3,Q3,fighandle3,xbar3,dx3,tp3,tpp3] = PQanalysis(t3_new,n3_new,1);

% Roll
[P4,Q4,fighandle4,xbar4,dx4,tp4,tpp4] = PQanalysis(t4_new,n4_new,1);

% Pitch
[P5,Q5,fighandle5,xbar5,dx5,tp5,tpp5] = PQanalysis(t5_new,n5_new,1);

% Yaw
[P6,Q6,fighandle6,xbar6,dx6,tp6,tpp6] = PQanalysis(t6_new,n6_new,1);

% % Considering positive peaks
% tp = dat2tp([t4_new,n4_new]);
% tp = tp(2:end,:);
% tpp = tp(tp(:,2)>0,:);
% hold on
% plot(tpp(:,1),tpp(:,2),'k.'); hold off
% 
% 
% 
% % find mean values and differences for positive peaks
% xbar = 0.5*( tpp(2:end-1,2)+tpp(1:end-2,2));
% dx = ( tpp(1:end-2,2)-tpp(2:end-1,2))./xbar;
% 
% pcoeffs = polyfit(xbar,dx,1);
% P = pcoeffs(2);
% Q = pcoeffs(1);
% 
% plot(xbar,dx,'.')
%     xlabel('$\bar{x}$','Interpreter','Latex')
%     ylabel('$\frac{x_i-x_{i+1}}{\bar{x}}$','Interpreter','Latex')
%     grid on
%     hold on
%     plot(xbar,polyval(pcoeffs,xbar))
%     legend('measured','fitted')

% Considering negative peaks
% tp = dat2tp([t5,n5]);
% tp = tp(2:end,:);
% tpn = tp(tp(:,2)<0,:);
% hold on
% plot(tpn(:,1),tpn(:,2),'r.'); hold off

% find mean values and differences for negative peaks
% xbar_n = 0.5*( tpn(2:end-1,2)+tpn(1:end-2,2));
% dx_n = ( tpn(1:end-2,2)-tpn(2:end-1,2))./xbar_n;
% 
% pcoeffs = polyfit(xbar_n,dx_n,1);
% P_n = pcoeffs(2);
% Q_n = pcoeffs(1);
% 
% plot(xbar_n,dx_n,'.')
%     xlabel('$\bar{x}$','Interpreter','Latex')
%     ylabel('$\frac{x_i-x_{i+1}}{\bar{x}}$','Interpreter','Latex')
%     grid on
%     hold on
%     plot(xbar_n,polyval(pcoeffs,xbar_n))
%     legend('measured','fitted')

%% Damped Natural Time periods

% Surge
T_d1 = (tpp1(end,1)-tpp1(1,1))/(size(tpp2,1)-1);
w_d1 = 2*pi/T_d1;

% Sway
T_d2 = (tpp2(end,1)-tpp2(1,1))/(size(tpp2,1)-1);
w_d2 = 2*pi/T_d2;

% Heave
T_d3 = (tpp3(end,1)-tpp3(1,1))/(size(tpp3,1)-1);
w_d3 = 2*pi/T_d3;

% Roll
T_d4 = (tpp4(end,1)-tpp4(1,1))/(size(tpp4,1)-1);
w_d4 = 2*pi/T_d4;

% Pitch
T_d5 = (tpp5(end,1)-tpp5(1,1))/(size(tpp5,1)-1);
w_d5 = 2*pi/T_d5;

% Yaw
T_d6 = (tpp6(end,1)-tpp6(1,1))/(size(tpp6,1)-1);
w_d6 = 2*pi/T_d6;

%% Restoring Coeff. 

t_start = 100;
ind_surge = sum(t1<=t_start);
ind_sway = sum(t2<=t_start);
ind_heave = sum(t3<=t_start);
ind_roll = sum(t4<=t_start);
ind_pitch = sum(t5<=t_start);
ind_yaw = sum(t6<=t_start);

F_exc = [2250e3 2250e3 1e7 530000e3 530000e3 1e7].';
eta_i = [mean(n1(ind_surge:2000)) mean(n2(ind_sway:2000)) mean(n3(ind_heave:2000)) mean(n4(ind_roll:2000))*pi/180 mean(n5(ind_pitch:2000))*pi/180 mean(n6(ind_yaw:2000))*pi/180].';

k = F_exc./eta_i;

%% Linear and Quadratic Damping

P = [P1;P2;P3;P4;P5;P6];
Q = [Q1;Q2;Q3;Q4;Q5;Q6];
w_d = [w_d1;w_d2;w_d3;w_d4;w_d5;w_d6];

b1 = zeros(size(P,1),1);
b2 = zeros(size(P,1),1);
for i=1:size(P,1)
    b1(i,1) = P(i,1)*k(i,1)/w_d(i,1)/pi;
    b2(i,1) = Q(i,1)*3*k(i,1)/8/w_d(i,1)/w_d(i,1);
end    

fprintf("The natural time periods are: \n Surge = %0.2f s \n Sway = %0.2f s \n Heave = %0.2f s \n Roll = %0.2f s \n Pitch = %0.2f s \n Yaw = %0.2f s \n",T_d1,T_d2,T_d3,T_d4,T_d5,T_d6)
fprintf("\n")
fprintf("The linear damping terms are: \n Surge = %0.2f Ns/m \n Sway = %0.2f Ns/m \n Heave = %0.2f Ns/m \n Roll = %0.2f Ns/m \n Pitch = %0.2f Ns/m \n Yaw = %0.2f Ns/m \n",b1(1,1),b1(2,1),b1(3,1),b1(4,1),b1(5,1),b1(6,1))
fprintf("\n")
fprintf("The quadratic damping terms are: \n Surge = %0.2f Ns^2/m^2 \n Sway = %0.2f Ns^2/m^2 \n Heave = %0.2f Ns^2/m^2 \n Roll = %0.2f Ns^2/m^2 \n Pitch = %0.2f Ns^2/m^2 \n Yaw = %0.2f Ns^2/m^2 \n",b2(1,1),b2(2,1),b2(3,1),b2(4,1),b2(5,1),b2(6,1))

%% Fore-Aft Tower BM at Tower Bottom

BM = readmatrix('F:\MSc Marine Technology NTNU\3rd Semester\TMR03 - Integrated Dynamic Analysis of Floating Wind Turbines\SIMA_Matlab Codes\Matlab Routines\Free Decay\Fore-Aft BM at Tower Base\Book1.xlsx');
xx = BM(:,1);
yy = BM(:,2)*10e-3;
figure(7)
subplot(2,1,1)
plot(xx,yy,'r-');
grid on
xlabel('Time [s]')
ylabel('Bending Moment (kN-m)')

start = 200/0.8;
end1 = 228/0.8;
t_range = (xx(start):0.8:xx(end1)).';
BM_range = zeros(size(t_range,1),1);
kk = 1;
for i = start:1:end1
    BM_range(kk,1) = BM(i,2);
    kk = kk+1;
end

% Counting Peaks
tp1 = dat2tp([t_range,BM_range]);
tp1 = tp1(2:end,:);
tpp = tp1(tp1(:,2)>0,:);
subplot(2,1,2)
plot(t_range,BM_range,'b');
hold on
plot(tpp(:,1),tpp(:,2),'r*');hold off
xlabel('Time [s]')
ylabel('Bending Moment (kN-m)')

% tp2 = dat2tp([BM(:,1),BM(:,2)*10e-3]);
% tp2 = tp2(2:end,:);
% tpn = tp2(tp2(:,2)<0,:);
% plot(tpn(:,1),tpn(:,2),'r.'); hold off

Tp = 2.4;
freq = 1/Tp;







