function xbp = bpass(x,dt,flow,fhigh)
%-----------------------------------------------------------------------------
% BPASS - Matlab function that will band-pass filter time series stored as 
%         columns in a matrix.
%
%     bp = bpass(x,dt,flow,fhigh)
%
%     x(1:nsamples,1:ntimeseries)= timeseries matrix
%     dt = time step (s)
%     flow and fhigh = lower and upper cut-off frequencies, respectively (Hz)
%     xbp = band-pass filtered time  series
% H. Lie, Marintek, 09.04.2002
% KEK, 2006-01-13: Minor changes
%-----------------------------------------------------------------------------
nsamples = length(x(:,1));
domega = 2*pi/(dt*nsamples);            % Frequency interval (rad/s)
%
% Make frequency array:
%
Omega = zeros(nsamples,1);
%
if ~rem(nsamples,2),                    % Even number of samples
   nh = nsamples/2;
   Omega(1:nh+1) = domega*(0:nh);
   Omega(nh+2:nsamples) = domega*(-nh+1:-1);
else
   nh = (nsamples-1)/2;
   Omega(1:nh+1) = domega*(0:nh); 
   Omega(nh+2:nsamples) = domega*(-nh:-1);
end
%
Fx = fft(x);                             % Discrete frequency transform
%
%---- Band-pass filter signals ----
%
% Indices corresponding to frequences to be filtered off:
%
Ioff = find(abs(Omega)<flow*2*pi | abs(Omega)>fhigh*2*pi);

Z = zeros(size(Ioff,1),size(x,2));       % Block of zeros to be inserted
Fx(Ioff,:) = Z;                          % Zero out unwanted components
%
xbp = real(ifft(Fx));                      % Transform back to time domain. 
                                         