% readTimeDomainResults  Reads in results from a single simulation 
% Rotor only

close all
clear all
clc

%% User-defined input

fold_r = 'C:\Users\Vignesh\OneDrive - NTNU\SIMA\Rotor15MW\Allwindspeed\564-20211004093621\Allwindspeed_';

for (i=1:12)
folder_r = strcat(fold_r,int2str(i))
prefix_r = 'sima';
hullBody_r = 1; % typically 1 
numForChan_r = 23; % this is the number of channels in the force results ...
% storage. You can find this in the key_sima_elmfor.txt file. 

%% READ SIMULATION OUTPUT
%==========================================================================
%%READ SIMULATION OUTPUT
    % Read the SIMO results text file to get the channel names and number
    % of steps
    [nchan_r, nts_r, dt_r, chanNames_r] = readSIMO_resultstext([folder_r '\results.txt']);
    % Read the binary file
    AA_r = read_simoresults([folder_r '\results.tda'],nts_r);
    sizeAA_r = size(AA_r);
    if (sizeAA_r(1)<nts_r || nts_r<1); disp('Unable to read SIMO results'); return; end;
    % Determine which channels to read for the platform motions, wave
    % elevation
    [chanMotions_r, chanWave_r] = getchannelNumbers(chanNames_r,hullBody_r);
    if (chanMotions_r(1)<1 || chanWave_r<1); disp('Unable to read SIMO results'); return; end;
    time_SIMO_r = AA_r(:,2);
    
    % summarize data in matrix
    PlatMotions_r = AA_r(:,chanMotions_r);
    wave_r = AA_r(:,chanWave_r);

    % Read the wind turbine results 
    fname_r = [folder_r '\' prefix_r '_witurb.bin'];
    CC_r = read_rifbin(fname_r,0,26);
    sizeCC_r = size(CC_r);
    Nt_r = sizeCC_r(1);  % get the number of time steps
    if Nt_r<2; disp('Unable to read RIFLEX wind turbine results'); return; end; 
    
    time_WT_r = CC_r(:,2);
    omega_r = CC_r(:,3)*pi/180; % convert from deg/s to rad/s
    genTq_r = CC_r(:,5); 
    genPwr_r = CC_r(:,6); %
    azimuth_r = CC_r(:,7); 
    HubWindX_r = CC_r(:,8);
    HubWindY_r = CC_r(:,9);
    HubWindZ_r = CC_r(:,10);
    AeroForceX_r = CC_r(:,11);
    AeroMomX_r = CC_r(:,14);
    Bl1Pitch_r = CC_r(:,17);
    Bl2Pitch_r = CC_r(:,18);
    Bl3Pitch_r = CC_r(:,19); 

    % Read the RIFLEX force output file
    fname_r = [folder_r '\' prefix_r '_elmfor.bin'];
    BB_r = read_rifbin(fname_r,Nt_r);
    sizeBB_r = size(BB_r);
    if(sizeBB_r(1)<1); disp('Unable to read RIFLEX force results'); return; end;  

%     time_RIFLEX = BB(:,2); 
%     % Tower Base
%     TowerBaseAxial  = BB(:,3);
%     TowerBaseTors   = BB(:,4);
%     TowerBaseBMY    = BB(:,5);
%     TowerBaseBMZ    = BB(:,7);
%     TowerBaseShearY = BB(:,9);
%     TowerBaseShearZ = BB(:,11);
% 
%     % Tower Top
%     TowerTopAxial  = BB(:,13);
%     TowerTopTors   = BB(:,14);
%     TowerTopBMY    = BB(:,16); % end 2
%     TowerTopBMZ    = BB(:,18);
%     TowerTopShearY = BB(:,20);
%     TowerTopShearZ = BB(:,22);    
%     
%     % Blade 1
%     bl1Axial        = BB(:,23);
%     bl1Tors         = BB(:,24);
%     bl1BMY          = BB(:,25);
%     bl1BMZ          = BB(:,27);
%     bl1ShearY       = BB(:,29);
%     bl1ShearZ       = BB(:,31);
%     
%     % Blade 2
%     bl2Axial        = BB(:,33);
%     bl2Tors         = BB(:,34);
%     bl2BMY          = BB(:,35);
%     bl2BMZ          = BB(:,37);
%     bl2ShearY       = BB(:,39);
%     bl2ShearZ       = BB(:,41);
%     
%     
%     clear('fid','fname','data','BB','AA','CC','sizeBB','chanNames','chanWave','dt')
% %%END READ SIMULATION OUTPUT
% 
% %==========================================================================
% 
% %% DESCRIPTION OF TIME SERIES available in the workspace
% % =========================================================================
% % Platform motions and wave elevation:
% %   Time variable:             time_SIMO (mx1)  : s
% %   Platform positions:        PlatMotions (mx6): m,m,m,deg,deg,deg
% %   Wave elevation at origin   wave(mx1)        : m 
% 
% % Wind turbine results : 
% %   Time variable:             time_RIFLEX (jx1): s
% %   Thrust (shaft x):          AeroForceX (jx1) : kN
% %   Aero. torque (about shaft):AeroMomX (jx1)   : kNm (rotor inertia NOT INCLUDED) 
% %   Wind speed (shaft x):      HubWindX (jx1)   : m/s (at hub pos)
% %   Wind speed (shaft y):      HubWindY (jx1)   : m/s (at hub pos)
% %   Wind speed (shaft z):      HubWindZ (jx1)   : m/s (at hub pos)
% %   Rotor speed:               omega (jx1)      : rad/s
% %   Generator power:           genPwr (jx1)     : kW (including efficiency)
% %   Generator torque:          genTq (jx1)      : kNm (HSS)
% %   Azimuth angle:             azimuth (jx1)    : deg 
% 
% % Internal force results at the tower base: 
% %   Time variable:              time_RIFLEX (jx1)       : s
% %   Axial force at tower base:  TowerBaseAxial (jx1)    : kN  (DOF1)
% %   Bending mom. Y at tow base: TowerBaseBMY (jx1)      : kNm (DOF3)
% %   Bending mom. Z at tow base: TowerBaseBMZ (jx1)      : kNm (DOF5)
% %   Shear force Y at tow base:  TowerBaseShearY (jx1)   : kN  (DOF7)
% %   Shear force Z at tow base:  TowerBaseShearZ (jx1)   : kN  (DOF9)
% %   Torsional moment tow base:  TowerBaseTors (jx1)     : kNm (DOF2)
% 
% % Internal force results at the blade root for blade 1: 
% %   Time variable:              time_RIFLEX (jx1)       : s
% %   Axial force at tower base:  bl1Axial (jx1)          : kN  (DOF1)
% %   Bending mom. Y at tow base: bl1BMY (jx1)            : kNm (DOF3)
% %   Bending mom. Z at tow base: bl1BMZ (jx1)            : kNm (DOF5)
% %   Shear force Y at tow base:  bl1ShearY (jx1)         : kN  (DOF7)
% %   Shear force Z at tow base:  bl1ShearZ (jx1)         : kN  (DOF9)
% %   Torsional moment tow base:  bl1Tors (jx1)           : kNm (DOF2)
% 
% % Internal force results at the blade root for blade 2: 
% %   Time variable:              time_RIFLEX (jx1)       : s
% %   Axial force at tower base:  bl2Axial (jx1)          : kN  (DOF1)
% %   Bending mom. Y at tow base: bl2BMY (jx1)            : kNm (DOF3)
% %   Bending mom. Z at tow base: bl2BMZ (jx1)            : kNm (DOF5)
% %   Shear force Y at tow base:  bl2ShearY (jx1)         : kN  (DOF7)
% %   Shear force Z at tow base:  bl2ShearZ (jx1)         : kN  (DOF9)
% %   Torsional moment tow base:  bl2Tors (jx1)           : kNm (DOF2)
% 
% % =========================================================================
% %% Post-processing
% %  Statistical analysis, fatigue calculation, etc
% %  Remember to cut out the first part of the time series
% %  Hint: 
          t_start_r = 450;
          indS_r = sum(time_SIMO_r<=t_start_r);
          surge_r = PlatMotions_r(:,1);
          roll_r =  PlatMotions_r(:,4);
          pitch_r = PlatMotions_r(:,5);
           
          Meansurge_r(i)      = mean(surge_r(indS_r:end));
          Meanroll_r(i)     = mean(roll_r(indS_r:end));
          Meanpitch_r(i)      = mean(pitch_r(indS_r:end));
          
          MeanRotorSpeed_r(i) = mean(omega_r(indS_r:end))*9.549;
          MeanThrust_r(i)    = mean(AeroForceX_r(indS_r:end));
          MeanTorque_r(i)    = mean(genTq_r(indS_r:end));
          MeanPower_r(i)      = mean(genPwr_r(indS_r:end));
          MeanBladePitch_r(i) = mean(Bl1Pitch_r(indS_r:end));
end

s_r = [4:2:10,10.6,12:2:24];
figure(1);
plot(s_r,Meansurge_r,'r')
xline(10.6,'-',{'Rated Wind speed'}) 
xlabel('Wind Speed (m/s)')
ylabel('Mean Surge (m)')
title('Mean Surge vs Wind speed (Rotor Only)')

figure(2);
plot(s_r,Meanpitch_r,'r')
xline(10.6,'-',{'Rated Wind speed'}) 
xlabel('Wind Speed (m/s)')
ylabel('Mean Pitch (deg)')
title('Mean Pitch vs Wind speed (Rotor Only)')

figure(3);
plot(s_r,Meanroll_r,'r')
xline(10.6,'-',{'Rated Wind speed'}) 
xlabel('Wind Speed (m/s)')
ylabel('Mean Roll (deg)')
title('Mean Roll vs Wind speed (Rotor Only)')

figure(4);
plot(s_r,MeanRotorSpeed_r,'r')
xline(10.6,'-',{'Rated Wind speed'}) 
xlabel('Wind Speed (m/s)')
ylabel('Mean Rotor Speed (rpm)')
title('Mean Rotor Speed vs Wind speed (Rotor Only)')

figure(5);
plot(s_r,MeanPower_r,'r')
xline(10.6,'-',{'Rated Wind speed'}) 
xlabel('Wind Speed (m/s)')
ylabel('Mean Rotor Speed (rpm)')
title('Mean Rotor Speed vs Wind speed (Rotor Only)')

figure(6);
plot(s_r,MeanThrust_r,'r')
xline(10.6,'-',{'Rated Wind speed'}) 
xlabel('Wind Speed (m/s)')
ylabel('Mean Thrust (N)')
title('Mean Thrust vs Wind speed (Rotor Only)')